@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: ephy-bookmarks-editor.c,v 1.97.2.1 2003/09/11 19:12:17 chpe Exp $
+ * $Id: ephy-bookmarks-editor.c,v 1.97.2.4 2003/10/17 12:06:20 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -684,10 +684,21 @@ ephy_bookmarks_editor_get_type (void)
 	return ephy_bookmarks_editor_type;
 }
 
+static void
+ephy_bookmarks_editor_show (GtkWidget *widget)
+{
+	EphyBookmarksEditor *editor = EPHY_BOOKMARKS_EDITOR (widget);
+
+	gtk_widget_grab_focus (editor->priv->search_entry);
+
+	GTK_WIDGET_CLASS (parent_class)->show (widget);
+}
+
 static void
 ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -697,6 +708,8 @@ ephy_bookmarks_editor_class_init (EphyBookmarksEditorClass *klass)
 	object_class->set_property = ephy_bookmarks_editor_set_property;
 	object_class->get_property = ephy_bookmarks_editor_get_property;
 
+	widget_class->show = ephy_bookmarks_editor_show;
+
 	g_object_class_install_property (object_class,
 					 PROP_BOOKMARKS,
 					 g_param_spec_object ("bookmarks",
