@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks.c,v 1.50 2003/06/18 18:06:02 mpeseng Exp $
+ *  $Id: ephy-bookmarks.c,v 1.59.2.1 2003/09/16 16:08:52 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -31,20 +31,26 @@
 #include "ephy-node-common.h"
 #include "ephy-toolbars-model.h"
 #include "ephy-bookmarks-export.h"
+#include "ephy-bookmarks-import.h"
+#include "ephy-autocompletion.h"
+#include "session.h"
 
 #include <string.h>
 #include <bonobo/bonobo-i18n.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 
-#define EPHY_BOOKMARKS_XML_VERSION "0.1"
-
+#define EPHY_BOOKMARKS_XML_VERSION "1.0"
+#define BOOKMARKS_SAVE_DELAY (3 * 1000)
 #define MAX_FAVORITES_NUM 10
 
 struct EphyBookmarksPrivate
 {
 	EphyToolbarsModel *toolbars_model;
 	gboolean init_defaults;
+	gboolean dirty;
+	guint save_timeout_id;
 	char *xml_file;
+	char *rdf_file;
 	EphyNodeDb *db;
 	EphyNode *bookmarks;
 	EphyNode *keywords;
@@ -58,7 +64,6 @@ typedef struct
 {
 	const char *title;
 	const char *location;
-	const char *smart_url;
 } EphyBookmarksBookmarkInfo;
 
 static const EphyBookmarksBookmarkInfo default_bookmarks [] =
@@ -204,26 +209,22 @@ ephy_bookmarks_autocompletion_source_init (EphyAutocompletionSourceIface *iface)
 static void
 ephy_bookmarks_init_defaults (EphyBookmarks *eb)
 {
-	int i, id;
-	EphyNode *node;
+	int i;
 
 	for (i = 0; i < n_default_topics; i++)
 	{
-		ephy_bookmarks_add_keyword (eb, default_topics[i]);
+		ephy_bookmarks_add_keyword (eb, _(default_topics[i]));
 	}
 
 	for (i = 0; i < n_default_bookmarks; i++)
 	{
-		ephy_bookmarks_add (eb, default_bookmarks[i].title,
-				    default_bookmarks[i].location);
+		EphyNode *bmk;
 
-		node = ephy_bookmarks_find_bookmark (eb, default_bookmarks[i].location);
-		if (node == NULL) break;
-		id = ephy_node_get_id (node);
-		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE, id);
+		bmk = ephy_bookmarks_add (eb, _(default_bookmarks[i].title),
+				          _(default_bookmarks[i].location));
+		ephy_toolbars_model_add_bookmark (eb->priv->toolbars_model, FALSE,
+						  ephy_node_get_id (bmk));
 	}
-
-	ephy_bookmarks_save (eb);
 }
 
 static void
@@ -313,6 +314,7 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 	xmlDocPtr doc;
 	xmlNodePtr root, child;
 	char *tmp;
+	gboolean result = TRUE;
 
 	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE)
 		return FALSE;
@@ -321,12 +323,19 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 	g_return_val_if_fail (doc != NULL, FALSE);
 
 	root = xmlDocGetRootElement (doc);
-
+	child = root->children;
+	
 	tmp = xmlGetProp (root, "version");
-	g_assert (tmp != NULL && strcmp (tmp, EPHY_BOOKMARKS_XML_VERSION) == 0);
+	if (tmp != NULL && strcmp (tmp, EPHY_BOOKMARKS_XML_VERSION) != 0)
+	{
+		g_warning ("Old bookmarks database format detected");
+
+		child = NULL;
+		result = FALSE;
+	}
 	g_free (tmp);
 
-	for (child = root->children; child != NULL; child = child->next)
+	for (; child != NULL; child = child->next)
 	{
 		EphyNode *node;
 
@@ -335,17 +344,17 @@ ephy_bookmarks_load (EphyBookmarks *eb)
 
 	xmlFreeDoc (doc);
 
-	return TRUE;
+	return result;
 }
 
-void
+static void
 ephy_bookmarks_save (EphyBookmarks *eb)
 {
 	xmlDocPtr doc;
 	xmlNodePtr root;
+	xmlNodePtr comment;
 	GPtrArray *children;
 	int i;
-	char *rdf_file;
 
 	LOG ("Saving bookmarks")
 
@@ -357,6 +366,9 @@ ephy_bookmarks_save (EphyBookmarks *eb)
 	xmlSetProp (root, "version", EPHY_BOOKMARKS_XML_VERSION);
 	xmlDocSetRootElement (doc, root);
 
+	comment = xmlNewDocComment (doc, "Do not rely on this file, it's only for internal use. Use bookmarks.rdf instead.");
+	xmlAddChild (root, comment);
+
 	children = ephy_node_get_children (eb->priv->keywords);
 	for (i = 0; i < children->len; i++)
 	{
@@ -388,11 +400,40 @@ ephy_bookmarks_save (EphyBookmarks *eb)
 	xmlFreeDoc(doc);
 
 	/* Export bookmarks in rdf */
-	rdf_file = g_build_filename (ephy_dot_dir (),
-				     "bookmarks.rdf",
-				     NULL);
-	ephy_bookmarks_export_rdf (eb, rdf_file);
-	g_free (rdf_file);
+	ephy_bookmarks_export_rdf (eb, eb->priv->rdf_file);
+}
+
+static gboolean
+save_bookmarks_delayed (EphyBookmarks *bookmarks)
+{
+	ephy_bookmarks_save (bookmarks);
+	bookmarks->priv->dirty = FALSE;
+	bookmarks->priv->save_timeout_id = 0;
+
+	return FALSE;
+}
+
+static void
+ephy_bookmarks_save_delayed (EphyBookmarks *bookmarks, int delay)
+{
+	if (!bookmarks->priv->dirty)
+	{
+		bookmarks->priv->dirty = TRUE;
+
+		if (delay > 0)
+		{
+			bookmarks->priv->save_timeout_id =
+				g_timeout_add (BOOKMARKS_SAVE_DELAY,
+				               (GSourceFunc) save_bookmarks_delayed,
+					       bookmarks);
+		}
+		else
+		{
+			bookmarks->priv->save_timeout_id =
+				g_idle_add ((GSourceFunc) save_bookmarks_delayed,
+					    bookmarks);
+		}
+	}
 }
 
 static double
@@ -478,7 +519,7 @@ update_favorites_menus ()
 	Session *session;
 	const GList *l;
 
-	session = ephy_shell_get_session (ephy_shell);
+	session = SESSION (ephy_shell_get_session (ephy_shell));
 	l = session_get_windows (session);
 
 	for (; l != NULL; l = l->next)
@@ -522,6 +563,7 @@ bookmarks_changed_cb (EphyNode *node,
 		      EphyBookmarks *eb)
 {
 	ephy_bookmarks_emit_data_changed (eb);
+	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
 }
 
 static void
@@ -532,6 +574,7 @@ bookmarks_removed_cb (EphyNode *node,
 {
 	ephy_bookmarks_emit_data_changed (eb);
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
+	ephy_bookmarks_save_delayed (eb, BOOKMARKS_SAVE_DELAY);
 }
 
 static char *
@@ -624,14 +667,20 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	GValue value = { 0, };
 	EphyNodeDb *db;
 
+	LOG ("Init");
+
         eb->priv = g_new0 (EphyBookmarksPrivate, 1);
 	eb->priv->toolbars_model = NULL;
 
-	db = ephy_node_db_new ("EphyBookmarks");
+	db = ephy_node_db_new (EPHY_NODE_DB_BOOKMARKS);
 	eb->priv->db = db;
-
+	eb->priv->dirty = FALSE;
+	eb->priv->save_timeout_id = 0;
 	eb->priv->xml_file = g_build_filename (ephy_dot_dir (),
-					       "bookmarks.xml",
+					       "ephy-bookmarks.xml",
+					       NULL);
+	eb->priv->rdf_file = g_build_filename (ephy_dot_dir (),
+					       "bookmarks.rdf",
 					       NULL);
 
 	/* Bookmarks */
@@ -703,7 +752,13 @@ ephy_bookmarks_init (EphyBookmarks *eb)
 	g_value_unset (&value);
 	ephy_node_add_child (eb->priv->keywords, eb->priv->notcategorized);
 
-	eb->priv->init_defaults = !ephy_bookmarks_load (eb);
+	if (!ephy_bookmarks_load (eb))
+	{
+		if (!ephy_bookmarks_import_rdf (eb, eb->priv->rdf_file))
+		{
+			eb->priv->init_defaults = !ephy_bookmarks_load (eb);
+		}
+	}
 
 	ephy_bookmarks_emit_data_changed (eb);
 
@@ -722,6 +777,11 @@ ephy_bookmarks_finalize (GObject *object)
 
         g_return_if_fail (eb->priv != NULL);
 
+	if (eb->priv->save_timeout_id != 0)
+	{
+		g_source_remove (eb->priv->save_timeout_id);
+	}
+
 	ephy_bookmarks_save (eb);
 
 	ephy_node_unref (eb->priv->bookmarks);
@@ -737,6 +797,9 @@ ephy_bookmarks_finalize (GObject *object)
 					      (gpointer *)&eb->priv->toolbars_model);
 	}
 
+	g_free (eb->priv->xml_file);
+	g_free (eb->priv->rdf_file);
+
         g_free (eb->priv);
 
 	LOG ("Bookmarks finalized")
@@ -801,6 +864,7 @@ ephy_bookmarks_add (EphyBookmarks *eb,
 
 	ephy_bookmarks_emit_data_changed (eb);
 	g_signal_emit (G_OBJECT (eb), ephy_bookmarks_signals[TREE_CHANGED], 0);
+	ephy_bookmarks_save_delayed (eb, 0);
 
 	return bm;
 }
@@ -821,13 +885,17 @@ ephy_bookmarks_set_address (EphyBookmarks *eb,
 	update_has_smart_address (bookmark, address);
 }
 
-EphyNode*
+EphyNode *
 ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 			      const char *url)
 {
 	GPtrArray *children;
 	int i;
 
+	g_return_val_if_fail (IS_EPHY_BOOKMARKS (eb), NULL);
+	g_return_val_if_fail (eb->priv->bookmarks != NULL, NULL);
+	g_return_val_if_fail (url != NULL, NULL);
+
 	children = ephy_node_get_children (eb->priv->bookmarks);
 	for (i = 0; i < children->len; i++)
 	{
@@ -837,7 +905,8 @@ ephy_bookmarks_find_bookmark (EphyBookmarks *eb,
 		kid = g_ptr_array_index (children, i);
 		location = ephy_node_get_property_string
 			(kid, EPHY_NODE_BMK_PROP_LOCATION);
-		if (strcmp (url, location) == 0)
+
+		if (location != NULL && strcmp (url, location) == 0)
 		{
 			ephy_node_thaw (eb->priv->bookmarks);
 			return kid;
@@ -1049,13 +1118,44 @@ ephy_bookmarks_add_keyword (EphyBookmarks *eb,
 	return key;
 }
 
-void 
+void
 ephy_bookmarks_remove_keyword (EphyBookmarks *eb,
-				    EphyNode *keyword)
+			       EphyNode *keyword)
 {
 	ephy_node_remove_child (eb->priv->keywords, keyword);
 }
 
+char *
+ephy_bookmarks_get_topic_uri (EphyBookmarks *eb,
+			      EphyNode *node)
+{
+	char *uri;
+
+	if (ephy_bookmarks_get_bookmarks (eb) == node)
+	{
+		uri = g_strdup ("topic://Special/All");
+	}
+	else if (ephy_bookmarks_get_not_categorized (eb) == node)
+	{
+		uri = g_strdup ("topic://Special/NotCategorized");
+	}
+	else if (ephy_bookmarks_get_favorites (eb) == node)
+	{
+		uri = g_strdup ("topic://Special/Favorites");
+	}
+	else
+	{
+		const char *name;
+
+		name = ephy_node_get_property_string
+			(node, EPHY_NODE_KEYWORD_PROP_NAME);
+
+		uri = g_strdup_printf ("topic://%s", name);
+	}
+
+	return uri;
+}
+
 EphyNode *
 ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 			     const char *name,
@@ -1064,9 +1164,11 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 	EphyNode *node;
 	GPtrArray *children;
 	int i;
+	const char *topic_name;
 
 	g_return_val_if_fail (name != NULL, NULL);
 
+	topic_name = name;
 
 	if (g_utf8_strlen (name, -1) == 0)
 	{
@@ -1074,6 +1176,23 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 		return NULL;
 	}
 
+	if (strcmp (name, "topic://Special/All") == 0)
+	{
+		return ephy_bookmarks_get_bookmarks (eb);
+	}
+	else if (strcmp (name, "topic://Special/NotCategorized") == 0)
+	{
+		return ephy_bookmarks_get_not_categorized (eb);
+	}
+	else if (strcmp (name, "topic://Special/Favorites") == 0)
+	{
+		return ephy_bookmarks_get_favorites (eb);
+	}
+	else if (g_str_has_prefix (name, "topic://"))
+	{
+		topic_name += strlen ("topic://");
+	}
+
 	children = ephy_node_get_children (eb->priv->keywords);
 	node = NULL;
 	for (i = 0; i < children->len; i++)
@@ -1084,8 +1203,8 @@ ephy_bookmarks_find_keyword (EphyBookmarks *eb,
 		 kid = g_ptr_array_index (children, i);
 		 key = ephy_node_get_property_string (kid, EPHY_NODE_KEYWORD_PROP_NAME);
 
-		 if ((partial_match && g_str_has_prefix (key, name) > 0) ||
-		     (!partial_match && strcmp (key, name) == 0))
+		 if ((partial_match && g_str_has_prefix (key, topic_name) > 0) ||
+		     (!partial_match && strcmp (key, topic_name) == 0))
 		 {
 			 node = kid;
 		 }
