@@ -42,7 +42,7 @@ enum
 	EPHY_NODE_BMK_PROP_LOCATION = 3,
 	EPHY_NODE_BMK_PROP_KEYWORDS = 4,
 	EPHY_NODE_KEYWORD_PROP_NAME = 5,
-	EPHY_NODE_BMK_PROP_SMART_LOCATION = 6,
+	EPHY_NODE_BMK_PROP_HAS_SMART_ADDRESS = 9,
 	EPHY_NODE_BMK_PROP_ICON = 7,
 	EPHY_NODE_KEYWORD_PROP_PRIORITY = 8
 };
@@ -57,24 +57,21 @@ struct EphyBookmarksClass
 {
         GObjectClass parent_class;
 
-	void (* bookmark_remove) (EphyBookmarks *eb,
-			          long id);
-	void (* topic_remove)    (EphyBookmarks *eb,
-			          long id);
+	void (* tree_changed) (EphyBookmarks *eb);
 };
 
 GType		ephy_bookmarks_get_type		(void);
 
 EphyBookmarks  *ephy_bookmarks_new		(void);
 
+EphyNode	*ephy_bookmarks_get_from_id     (EphyBookmarks *eb,
+						 long id);
+
 /* Bookmarks */
 
-void		ephy_bookmarks_save		(EphyBookmarks *eb);
-
-EphyNode       *ephy_bookmarks_add		(EphyBookmarks *eb,
+EphyNode        *ephy_bookmarks_add		(EphyBookmarks *eb,
 						 const char *title,
-						 const char *url,
-						 const char *smart_url);
+						 const char *url);
 
 EphyNode*	 ephy_bookmarks_find_bookmark   (EphyBookmarks *eb,
 						 const char *url);
@@ -83,6 +80,10 @@ void		 ephy_bookmarks_set_icon	(EphyBookmarks *eb,
 						 const char *url,
 						 const char *icon);
 
+void		 ephy_bookmarks_set_address     (EphyBookmarks *eb,
+			                         EphyNode *bookmark,
+			                         const char *address);
+
 char		*ephy_bookmarks_solve_smart_url (EphyBookmarks *eb,
 						 const char *smart_url,
 						 const char *content);
@@ -111,6 +112,9 @@ void            ephy_bookmarks_unset_keyword    (EphyBookmarks *eb,
 						 EphyNode *keyword,
 						 EphyNode *bookmark);
 
+char           *ephy_bookmarks_get_topic_uri    (EphyBookmarks *eb,
+			                         EphyNode *node);
+
 /* Favorites */
 
 EphyNode       *ephy_bookmarks_get_favorites	(EphyBookmarks *eb);
@@ -121,6 +125,8 @@ EphyNode       *ephy_bookmarks_get_keywords	(EphyBookmarks *eb);
 
 EphyNode       *ephy_bookmarks_get_bookmarks	(EphyBookmarks *eb);
 
+EphyNode       *ephy_bookmarks_get_not_categorized (EphyBookmarks *eb);
+
 G_END_DECLS
 
 #endif
